//
//  StepperController.m
//  Stepper
//
//  Created by Phidgets on 2015-07-22.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "StepperController.h"

@implementation StepperController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetStepper_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetStepper_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetStepper_setOnStoppedHandler(ch, gotStoppedData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetStepper_setOnPositionChangeHandler(ch, gotPositionChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetStepper_setOnVelocityChangeHandler(ch, gotVelocityChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
    [tabBox setDelegate:self];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}


#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}
static void gotPositionChangeData(PhidgetStepperHandle phid, void *context, double position){
        [(__bridge id)context performSelectorOnMainThread:@selector(onPositionChangeHandler:)
                                               withObject:[NSNumber numberWithDouble:position]
                                            waitUntilDone:NO];
}
static void gotStoppedData(PhidgetStepperHandle phid, void *context){
        [(__bridge id)context performSelectorOnMainThread:@selector(onStoppedHandler)
                                               withObject:nil
                                            waitUntilDone:NO];
}
static void gotVelocityChangeData(PhidgetStepperHandle phid, void *context, double velocity){
        [(__bridge id)context performSelectorOnMainThread:@selector(onVelocityChangeHandler:)
                                               withObject:[NSNumber numberWithDouble:velocity]
                                            waitUntilDone:NO];
}


#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    int heightChange = 475;
    double minVelocityLimit, maxVelocityLimit, velocityLimit;
    double minCurrentLimit, maxCurrentLimit, currentLimit;
    double minAcceleration, maxAcceleration, acceleration;
    double minPosition = -100000, maxPosition = 100000, targetPosition;
    int engaged;
    Phidget_DeviceID deviceID;
    
    //Get information from channel that will allow us to configure the GUI properly
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    PhidgetStepper_getMinVelocityLimit(ch, &minVelocityLimit);
    PhidgetStepper_getMaxVelocityLimit(ch, &maxVelocityLimit);
    PhidgetStepper_getVelocityLimit(ch, &velocityLimit);
    PhidgetStepper_getMaxCurrentLimit(ch, &maxCurrentLimit);
    PhidgetStepper_getMinCurrentLimit(ch, &minCurrentLimit);
    PhidgetStepper_getMinAcceleration(ch, &minAcceleration);
    PhidgetStepper_getMaxAcceleration(ch, &maxAcceleration);
    PhidgetStepper_getTargetPosition(ch, &targetPosition);
    PhidgetStepper_getAcceleration(ch, &acceleration);
    PhidgetStepper_getCurrentLimit(ch, &currentLimit);
    PhidgetStepper_getEngaged(ch, &engaged);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [motorStatusBox setHidden:NO];
    [tabBox setHidden:NO];
    
    //set limits for various controls based on information read from channel
    [positionVelocityLimitSlider setMinValue:minVelocityLimit];
    [positionVelocityLimitSlider setMaxValue:maxVelocityLimit];
    [positionVelocityLimitSlider setDoubleValue:velocityLimit];
    [positionMinVelocityLimitLabel setStringValue:[NSString stringWithFormat:@"%d",(int)minVelocityLimit]];
    [positionMaxVelocityLimitLabel setStringValue:[NSString stringWithFormat:@"%d",(int)maxVelocityLimit]];
    [positionVelocityLimitText setStringValue:[NSString stringWithFormat:@"%d",(int)velocityLimit]];
    
    [velocityVelocityLimitSlider setMinValue:-maxVelocityLimit];
    [velocityVelocityLimitSlider setMaxValue:maxVelocityLimit];
    [velocityVelocityLimitSlider setDoubleValue:velocityLimit];
    [velocityMinVelocityLimitLabel setStringValue:[NSString stringWithFormat:@"%d",(int)-maxVelocityLimit]];
    [velocityMaxVelocityLimitLabel setStringValue:[NSString stringWithFormat:@"%d",(int)maxVelocityLimit]];
    [velocityVelocityLimitText setStringValue:[NSString stringWithFormat:@"%d",(int)velocityLimit]];
    
    if(deviceID != PHIDID_1062){
        [positionCurrentLimitBox setHidden:NO];
        [velocityCurrentyLimitBox setHidden:NO];
        [positionCurrentLimitSlider setMinValue:0.0];
        [positionCurrentLimitSlider setMaxValue:maxCurrentLimit];
        [positionMinCurrentLimitLabel setStringValue:@"0 A"];
        [positionMaxCurrentLimitLabel setStringValue:[NSString stringWithFormat:@"%.3f A",maxCurrentLimit]];
        [positionCurrentLimitSlider setDoubleValue:currentLimit];
        [positionCurrentLimitText setStringValue:[NSString stringWithFormat:@"%.3f A", currentLimit]];
        [velocityCurrentLimitSlider setMinValue:0.0];
        [velocityCurrentLimitSlider setMaxValue:maxCurrentLimit];
        [velocityMinCurrentLimitLabel setStringValue:@"0 A"];
        [velocityMaxCurrentLimitLabel setStringValue:[NSString stringWithFormat:@"%.3f A",maxCurrentLimit]];
        [velocityCurrentLimitSlider setDoubleValue:currentLimit];
        [velocityCurrentLimitText setStringValue:[NSString stringWithFormat:@"%.3f A", currentLimit]];
    }
    else{
        [positionCurrentLimitBox setHidden:YES];
        [velocityCurrentyLimitBox setHidden:YES];
    }
    
    if(deviceID == PHIDID_STC1000){
        [positionHoldingCurrentLimitBox setHidden:NO];
        [velocityHoldingCurrentLimitBox setHidden:NO];
    }
    else{
        [positionHoldingCurrentLimitBox setHidden:YES];
        [velocityHoldingCurrentLimitBox setHidden:YES];
    }
    
    [positionAccelerationSlider setMinValue:minAcceleration];
    [positionAccelerationSlider setMaxValue:maxAcceleration];
    [positionAccelerationText setDoubleValue:acceleration];
    [positionMinAccelerationLabel setStringValue:[NSString stringWithFormat:@"%d",(int)minAcceleration]];
    [positionMaxAccelerationLabel setStringValue:[NSString stringWithFormat:@"%d",(int)maxAcceleration]];
    [positionAccelerationText setStringValue:[NSString stringWithFormat:@"%d",(int)acceleration]];
    
    [velocityAccelerationSlider setMinValue:minAcceleration];
    [velocityAccelerationSlider setMaxValue:maxAcceleration];
    [velocityAccelerationSlider setDoubleValue:acceleration];
    [velocityAccelerationText setStringValue:[NSString stringWithFormat:@"%d",(int)acceleration]];
    [velocityMinAccelerationLabel setStringValue:[NSString stringWithFormat:@"%d",(int)minAcceleration]];
    [velocityMaxAccelerationLabel setStringValue:[NSString stringWithFormat:@"%d",(int)maxAcceleration]];
    [velocityAccelerationText setStringValue:[NSString stringWithFormat:@"%d",(int)acceleration]];
    
    [positionTargetPositionSlider setMinValue:minPosition];
    [positionTargetPositionSlider setMaxValue:maxPosition];
    [positionMinTargetPositionLabel setStringValue:[NSString stringWithFormat:@"%.0f",minPosition]];
    [positionMaxTargetPositionLabel setStringValue:[NSString stringWithFormat:@"%.0f",maxPosition]];
    [positionTargetPositionSlider setDoubleValue:targetPosition];
    [positionTargetPositionText setStringValue:[NSString stringWithFormat:@"%.0f",targetPosition]];
    
    if(engaged){
        engagedButton.title = @"Disengage";
    }
    else{
        engagedButton.title = @"Engage";
    }
    

    //Adjusting view height
    NSRect frame = [mainWindow frame];
    frame.size.width = 550;
    frame.size.height += heightChange;
    frame.origin.y -= heightChange;
    [mainWindow setFrame:frame display:NO animate:NO];
    [mainWindow center];

}

- (void)onDetachHandler{
    //Reset
    [motorPositionText setStringValue:@"N/A"];
    [motorVelocityText setStringValue:@"N/A"];
    [rescaleTextField setStringValue:@"1"];
    [positionAccelerationText setStringValue:@"N/A"];
    [positionVelocityLimitText setStringValue:@"N/A"];
    [positionTargetPositionText setStringValue:@"N/A"];
    [positionCurrentLimitText setStringValue:@"N/A"];
    [velocityAccelerationText setStringValue:@"N/A"];
    [velocityVelocityLimitText setStringValue:@"N/A"];
    [velocityCurrentLimitText setStringValue:@"N/A"];
    
    [motorStatusBox setHidden:YES];
    [tabBox setHidden:YES];
    [positionHoldingCurrentLimitBox setHidden:YES];
    [velocityHoldingCurrentLimitBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)addError:(PhidgetReturnCode)result{
    const char *errorString;
    Phidget_getErrorDescription(result, &errorString);
    [self outputError:errorString];
}


- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

- (void)onPositionChangeHandler:(NSNumber *)position{
    [motorPositionText setStringValue:[NSString stringWithFormat:@"%.3f",[position doubleValue]]];
    [self updateStopped];
}

- (void)onVelocityChangeHandler:(NSNumber *)velocity{
    [motorVelocityText setStringValue:[NSString stringWithFormat:@"%.3f",[velocity doubleValue]]];
    [self updateStopped];
}

- (void)onStoppedHandler
{
    [self updateStopped];
}

-(void)updateStopped{
    int isMoving, engaged;
    PhidgetReturnCode result;
    result = PhidgetStepper_getEngaged(ch, &engaged);
    if(!engaged)
        return;
    result = PhidgetStepper_getIsMoving(ch, &isMoving);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    
    if(isMoving){
        [motorMovingLabel setStringValue:@"Moving"];
        [motorMovingLabel setBackgroundColor:[NSColor colorWithRed:0 green:0.561 blue:0 alpha:1]];
    }
    else{
        [motorMovingLabel setStringValue:@"Stopped"];
        [motorMovingLabel setBackgroundColor:[NSColor  colorWithRed:1.0 green:0.494 blue:0.475 alpha:1]];
    }
}

#pragma mark GUI controls

- (void)tabView:(NSTabView *)tabView
didSelectTabViewItem:(NSTabViewItem *)tabViewItem{
    if([[tabViewItem identifier] intValue] == 1){
        PhidgetStepper_setControlMode(ch, CONTROL_MODE_STEP);
    }
    else{
        PhidgetStepper_setControlMode(ch, CONTROL_MODE_RUN);
    }
    
}

- (IBAction)engaged:(NSButton *)sender {
    PhidgetReturnCode result;
    int engaged;
    PhidgetStepper_getEngaged(ch, &engaged);
    if(engaged){
        result = PhidgetStepper_setEngaged(ch, 0);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
        [sender setTitle:@"Engage"];
    }
    else{
        result = PhidgetStepper_setEngaged(ch, 1);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
        [sender setTitle:@"Disengage"];
    }
}


- (IBAction)setRescaleFactor:(id)sender{
    PhidgetReturnCode result;
    double minAcceleration, maxAcceleration, minVelocityLimit, maxVelocityLimit, minPosition, maxPosition, position, velocityLimit, acceleration, rescaleFactor;
    PhidgetStepper_ControlMode controlMode;
    rescaleFactor = [rescaleTextField doubleValue];
    result = PhidgetStepper_setRescaleFactor(ch, rescaleFactor);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    PhidgetStepper_getAcceleration(ch, &acceleration);
    PhidgetStepper_getVelocityLimit(ch, &velocityLimit);
    PhidgetStepper_getPosition(ch, &position);
    PhidgetStepper_getMinAcceleration(ch, &minAcceleration);
    PhidgetStepper_getMaxAcceleration(ch, &maxAcceleration);
    PhidgetStepper_getMinVelocityLimit(ch, &minVelocityLimit);
    PhidgetStepper_getMaxVelocityLimit(ch, &maxVelocityLimit);
    PhidgetStepper_getControlMode(ch, &controlMode);
    if(controlMode == CONTROL_MODE_STEP){
        /*PhidgetStepper_getMinPosition(ch, &minPosition);
        PhidgetStepper_getMaxPosition(ch, &maxPosition);*/
        minPosition = -10000 * rescaleFactor;
        maxPosition = 10000 * rescaleFactor;
        [positionTargetPositionText setStringValue:@"0"];
        [positionTargetPositionSlider setDoubleValue:0.0];
        [positionMinTargetPositionLabel setStringValue:[NSString stringWithFormat:@"%f",minPosition]];
        [positionMaxTargetPositionLabel setStringValue:[NSString stringWithFormat:@"%f",maxPosition]];
        [positionTargetPositionSlider setMinValue:minPosition];
        [positionTargetPositionSlider setMaxValue:maxPosition];
    }
    [positionAccelerationText setStringValue:[NSString stringWithFormat:@"%f",acceleration]];
    [positionAccelerationSlider setDoubleValue:acceleration];
    [positionMinAccelerationLabel setStringValue:[NSString stringWithFormat:@"%f",minAcceleration]];
    [positionMaxAccelerationLabel setStringValue:[NSString stringWithFormat:@"%f", maxAcceleration]];
    [positionAccelerationSlider setMinValue:minAcceleration];
    [positionAccelerationSlider setMaxValue:maxAcceleration];
    [positionVelocityLimitText setStringValue:[NSString stringWithFormat:@"%f", velocityLimit]];
    [positionVelocityLimitSlider setDoubleValue:velocityLimit];
    [positionMinVelocityLimitLabel setStringValue:[NSString stringWithFormat:@"%f", minVelocityLimit]];
    [positionMaxVelocityLimitLabel setStringValue:[NSString stringWithFormat:@"%f",maxVelocityLimit]];
    [positionVelocityLimitSlider setMinValue:minVelocityLimit];
    [positionVelocityLimitSlider setMaxValue:maxVelocityLimit];
    [velocityAccelerationText setStringValue:[NSString stringWithFormat:@"%f", acceleration]];
    [velocityAccelerationSlider setDoubleValue:acceleration];
    [velocityMinAccelerationLabel setStringValue:[NSString stringWithFormat:@"%f", minAcceleration]];
    [velocityMaxAccelerationLabel setStringValue:[NSString stringWithFormat:@"%f", maxAcceleration]];
    [velocityAccelerationSlider setMinValue:minAcceleration];
    [velocityAccelerationSlider setMaxValue:maxAcceleration];
    [velocityVelocityLimitText setStringValue:[NSString stringWithFormat:@"%f", velocityLimit]];
    [velocityVelocityLimitSlider setDoubleValue:velocityLimit];
    [velocityMaxVelocityLimitLabel setStringValue:[NSString stringWithFormat:@"%f", maxVelocityLimit]];
    [velocityMinVelocityLimitLabel setStringValue:[NSString stringWithFormat:@"%f", minVelocityLimit]];
    [velocityVelocityLimitSlider setMinValue:minVelocityLimit];
    [velocityVelocityLimitSlider setMaxValue:maxVelocityLimit];
}

//Position tab events
- (IBAction)setHoldingCurrentLimit:(id)sender {
    PhidgetReturnCode result;
    NSButton *button = (NSButton *)sender;
    double holdingCurrent;
    if(button.tag == 0)
        holdingCurrent = [positionHoldingCurrentTextField doubleValue];
    else
        holdingCurrent = [velocityHoldingCurrentLimitTextField doubleValue];
    
    result = PhidgetStepper_setHoldingCurrentLimit(ch, holdingCurrent);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    
}

- (IBAction)positionSetMinMaxTargetPosition:(id)sender {
    PhidgetReturnCode result;
    double minPosition, maxPosition;
    double absValue = fabs([positionMinMaxTargetPositionTextField doubleValue]);
    result = PhidgetStepper_getMinPosition(ch, &minPosition);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    result = PhidgetStepper_getMaxPosition(ch, &maxPosition);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    if(absValue > maxPosition){
        [positionMinMaxTargetPositionTextField setStringValue:@"Invalid"];
        return;
    }
    [positionTargetPositionSlider setMinValue:-absValue];
    [positionTargetPositionSlider setMaxValue:absValue];
    [positionMinTargetPositionLabel setStringValue:[NSString stringWithFormat:@"%f",-absValue]];
    [positionMaxTargetPositionLabel setStringValue:[NSString stringWithFormat:@"%f",absValue]];
}

- (IBAction)positionSetTargetPosition:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetStepper_setTargetPosition(ch, [positionTargetPositionSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [positionTargetPositionText setStringValue:[NSString stringWithFormat:@"%f",[positionTargetPositionSlider doubleValue]]];
    [self updateStopped];
}

- (IBAction)positionSetCurrentLimit:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetStepper_setCurrentLimit(ch, [positionCurrentLimitSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [positionCurrentLimitText setStringValue:[NSString stringWithFormat:@"%.3f A",[positionCurrentLimitSlider doubleValue]]];
}

- (IBAction)positionSetAcceleration:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetStepper_setAcceleration(ch, [positionAccelerationSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [positionAccelerationText setStringValue:[NSString stringWithFormat:@"%f",[positionAccelerationSlider doubleValue]]];
}

- (IBAction)positionSetVelocityLimit:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetStepper_setVelocityLimit(ch, [positionVelocityLimitSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [positionVelocityLimitText setStringValue:[NSString stringWithFormat:@"%f",[positionVelocityLimitSlider doubleValue]]];
}

- (IBAction)positionZero:(id)sender{
    double position;
    PhidgetReturnCode result;
    PhidgetStepper_getPosition(ch, &position);
    result = PhidgetStepper_addPositionOffset(ch, -position);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetStepper_getPosition(ch, &position);
    
    [motorPositionText setStringValue:[NSString stringWithFormat:@"%f", position]];
}

//velocity tab events
- (IBAction)velocitySetAcceleration:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetStepper_setAcceleration(ch, [velocityAccelerationSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [velocityAccelerationText setStringValue:[NSString stringWithFormat:@"%f",[velocityAccelerationSlider doubleValue]]];
}

- (IBAction)velocitySetVelocityLimit:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetStepper_setVelocityLimit(ch, [velocityVelocityLimitSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [velocityVelocityLimitText setStringValue:[NSString stringWithFormat:@"%f",[velocityVelocityLimitSlider doubleValue]]];
}

- (IBAction)velocitySetCurrentLimit:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetStepper_setCurrentLimit(ch, [velocityCurrentLimitSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [velocityCurrentLimitText setStringValue:[NSString stringWithFormat:@"%.3f A",[velocityCurrentLimitSlider doubleValue]]];
}

- (IBAction)velocityStop:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetStepper_setVelocityLimit(ch, 0);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [velocityVelocityLimitText setStringValue:@"0"];
    [velocityVelocityLimitSlider setDoubleValue:0.0];
}

@end
